/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Suffix Array based String Kernel.
 *
 * The Initial Developer of the Original Code is
 * Statistical Machine Learning Program (SML), National ICT Australia (NICTA).
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *   Choon Hui Teo <ChoonHui.Teo@rsise.anu.edu.au>
 *   S V N Vishwanathan <SVN.Vishwanathan@nicta.com.au>
 *
 * ***** END LICENSE BLOCK ***** */


// File    : sask/Code/ltp-sa-nsl.cpp
//
// Authors : Choon Hui Teo      (ChoonHui.Teo@rsise.anu.edu.au)
//           S V N Vishwanathan (SVN.Vishwanathan@nicta.com.au)
//
// Created :    Feb 2006
//
// Updated : 24 Apr 2006
//           24 Oct 2006


//' Experiment : Prediction using direct kernel evaluation, that is, construct SK(x)
//'              then for all y, Compute_K(y). After that, multiplied with alphas.

#include "DataType.h"
#include "ErrorCode.h"
#include "ESA.h"
#include "StringKernel.h"

#include <fstream>
#include <sys/time.h>


int main()
{
	UInt32 datasize = 10000; 
	std::string textfilename = "../Data/bible.data"; //'size = 30383
	std::string patternfilename = "../Data/peter16.txt"; //' Peter Pan
	std::string runtimefilename = "direct-runtime-sa-nsl.log";
	std::string alltimefilename = "direct-alltime-sa-nsl.log";
  int swf = CONSTANT;
  Real swfParam = 0.0; //dummy for CONSTANT

  std::vector<std::string> data_vec(datasize);
	std::vector<UInt32> data_len(datasize);

  int counter = datasize;
	int total = 0;
	std::string line = "";
  std::string data = "";
	std::string pattern = "";
  
  ifstream text_fp;
	ifstream pattern_fp;
	ofstream runtime_fp;
	ofstream alltime_fp;
	ofstream result_fp;

	int n_iter = 10;
	int n_size = 10;
	
	int sizes[] = {10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000};

	timeval runtime_start, alltime_start, time_end;

	double runtime = 0.0;
	double alltime = 0.0;

	

	//' Open text file and read data
  text_fp.open(textfilename.c_str());
  if(!text_fp){
    std::cout<< "Unable to open input file : "<< textfilename << std::endl;
    exit(1);
  }
	while(counter && !text_fp.eof()){
		//' Load data
		std::getline(text_fp, line);

		//' Append sentinel char (e.g. '\n')
		line = line + SENTINEL;

		data_len[datasize -counter] = line.size();
		total += line.size();
    data_vec[datasize -counter] = line;
    counter--;
  }
  assert(!counter);
  text_fp.close();


	//' Open pattern file and read data
	pattern_fp.open(patternfilename.c_str());
	if(!pattern_fp) {
		std::cout << "Unable to open pattern file : " << patternfilename << std::endl;
		exit(1);
	}
	
	while(! pattern_fp.eof()) {
		getline(pattern_fp,line);
		pattern += line + ' ';  //' replace '\n' with whitespace
	}
	std::cout << "pattern size : " << pattern.size() << std::endl;
	pattern_fp.close();

	
	//' Open runtime file
	runtime_fp.open(runtimefilename.c_str());
	if(!runtime_fp) {
		std::cout<<"Unable to open output file : " << runtimefilename << std::endl;
		exit(1);
	}
	runtime_fp.setf(ios::fixed);

	//' Open alltime file
	alltime_fp.open(alltimefilename.c_str());
	if(!alltime_fp) {
		std::cout<<"Unable to open output file : " << alltimefilename << std::endl;
		exit(1);
	}
	alltime_fp.setf(ios::fixed);



	//' Experiment
	for(int i=0; i < n_size; i++) {

		std::cout << "\n Number of Support Vectors : " << sizes[i] << std::endl;
		
		Real kval = 0;
		Real sum = 0;
		Real dummyAlpha = 1;
		runtime = 0;  //' running time for this iteration

		//' Iteration...
		for(int j=0; j<n_iter; j++) {
			
			std::cout << "\n\nIteration : " << j << std::endl;

			for(int idx_sv=0; idx_sv < sizes[i]; idx_sv++) {
				
				//' Record starting alltime
				gettimeofday(&alltime_start, NULL);
				
				//' StringKernel contruction and preparation
				StringKernel sk(data_vec[idx_sv].size(), (SYMBOL*)data_vec[idx_sv].c_str(), swf, swfParam);
				sk.Set_Lvs();
				sk.PrecomputeVal();
							
				//' Record start time
				gettimeofday(&runtime_start, NULL);

				//' Kernel evaluation
				sk.Compute_K((SYMBOL*)pattern.c_str(),	pattern.length(), kval);

				//' Record end time
				gettimeofday(&time_end, NULL);

				
				if(j == 0) {
					//' count only once...
					sum += kval*dummyAlpha;  //' Simulate actual computation...
				}

				//' Calculate time elapsed (in milliseconds)
				double runtimeElapsed = (time_end.tv_sec - runtime_start.tv_sec)*1000;
				runtimeElapsed += double(time_end.tv_usec - runtime_start.tv_usec)/1000;
				
				double alltimeElapsed = (time_end.tv_sec - alltime_start.tv_sec)*1000;
				alltimeElapsed += double(time_end.tv_usec - alltime_start.tv_usec)/1000;
				
				runtime +=  runtimeElapsed;
				alltime +=  alltimeElapsed;
				
				std::cout << "[runtime] : " << runtimeElapsed << " ms.   " 
									<< "[alltime] : " << alltimeElapsed << " ms. " <<std::endl;
			}
		}
		//' Dump #runtime into runtime_fp
		runtime_fp << sizes[i] << " " << runtime/n_iter << endl;

		//' Dump #alltime into alltime_fp
		alltime_fp << sizes[i] << " " << alltime/n_iter << endl;
	}

	
	runtime_fp.close();
	alltime_fp.close();

	return 0;
}

