/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Suffix Array based String Kernel.
 *
 * The Initial Developer of the Original Code is
 * Statistical Machine Learning Program (SML), National ICT Australia (NICTA).
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *   Choon Hui Teo <ChoonHui.Teo@rsise.anu.edu.au>
 *   S V N Vishwanathan <SVN.Vishwanathan@nicta.com.au>
 *
 * ***** END LICENSE BLOCK ***** */


// File    : sask/Code/fixedpattern-sa-sl.cpp
//
// Authors : Choon Hui Teo      (ChoonHui.Teo@rsise.anu.edu.au)
//           S V N Vishwanathan (SVN.Vishwanathan@nicta.com.au)
//
// Created : 09 Feb 2006
//
// Updated : 24 Apr 2006
//           24 Oct 2006


//' Experiment : Kernel evaluation time. Fixed pattern size (128KB) vs varying text sizes (1KB - 8MB)
//' Subject    : sa-sl
//' Dataset    : Gutenberg


//' C/C++ header files
#include <cassert>
#include <cstdio>
#include <ctime>
#include <fstream>
#include <iostream>
#include <string>
#include <vector>
#include <sys/time.h>

//' Suffix arrays header files
#include "DataType.h"
#include "ErrorCode.h"
#include "ESA.h"
#include "StringKernel.h"


using namespace std;


int main()
{
	//' Recording kernel computation time w.r.t text and pattern sizes

	string textfilename    = "../Data/gutenberg.part1.txt";
	string patternfilename = "../Data/gutenberg.part2.txt";
	string logfilename     = "fixedpattern-sa-sl.log";
  int swf = CONSTANT;
  Real swfParam = 0.0; //dummy for CONSTANT

	//' Note that the text and pattern file sizes are larger than 10 millions
	
	int n_size = 14;
	int sizes[] = {1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072, 262144, 
								 524288, 1048576, 2097152, 4194304, 8388608};

	//' text, pattern and log files
	ifstream text_fp(textfilename.c_str());
	ifstream pattern_fp(patternfilename.c_str());
	ofstream log_fp(logfilename.c_str());

	char *text;
	char *pattern;

	Real dummyVal;
	timeval start, end;

	double runtime = 0;
	
	int n_iter = 10;

	
	if(!text_fp) {
		cout << "Error opening text file : " << textfilename << endl;
		exit(1);
	}
	if(!pattern_fp) {
		cout << "Error opening pattern file : " << patternfilename << endl;
		exit(1);
	}
	if(!log_fp) {
		cout << "Error opening log file : " << logfilename << endl;
		exit(1);
	}
	
	
	//' Fix pattern size at 128KB, that is sizes[7]
	int idx_fixed = 7;
	pattern = new char[sizes[idx_fixed]+1];
	pattern_fp.read(pattern,sizes[idx_fixed]);
	pattern[sizes[idx_fixed]-1] = SENTINEL;  //' enforce sentinel char,'\n'
	pattern[sizes[idx_fixed]] = '\0';  //' EOL
	

	//' For different text size
	for(int i=0; i<n_size; i++) {
		text = new char[sizes[i]+1];
		text_fp.read(text,sizes[i]);
		text[sizes[i]-1] = SENTINEL;  //' enforce sentinel char,'\n'
		text[sizes[i]] = '\0';  //' EOL

		//' Construct ESASK
		StringKernel sk(sizes[i], (SYMBOL*)text, swf, swfParam);
		sk.Set_Lvs();
		sk.PrecomputeVal();
	
		cout << "text : " << sizes[i] << "   pattern : " << sizes[idx_fixed] << endl;
		runtime = 0;
	
		for(int cnt=0; cnt<n_iter; cnt++) {
			//' Record start time
			gettimeofday(&start, NULL);
			
			sk.Compute_K((SYMBOL*)pattern, sizes[idx_fixed], dummyVal);
			
			//' Record end time
			gettimeofday(&end, NULL);
			
			//' Calculate time elapsed (in milliseconds)
			double timeElapsed = (end.tv_sec - start.tv_sec)*1000;
			timeElapsed += double(end.tv_usec - start.tv_usec)/1000;
			
			runtime +=  timeElapsed;
			cout << "runtime : " << timeElapsed << " ms" << endl;
		}
		//' Dump runtime into log file
		log_fp << sizes[i] << " " << runtime/n_iter << endl;

		//' Clear text
		delete text;
		text_fp.seekg(ios::beg);  //' rewind text file
	}
	
	text_fp.close();
	pattern_fp.close();
	log_fp.close();
	
	return 0;
}
